import json

from djsailthru.mail.message import SailthruEmailMessage
from djsailthru.tests.mock_backend import SailthruBackendMockTestCase

from sailthru.sailthru_error import SailthruClientError


class TestSailthruEamilMessage(SailthruBackendMockTestCase):
    def setUp(self):
        super(TestSailthruEamilMessage, self).setUp()
        self.message = SailthruEmailMessage('foo@bar.com', 'Email Template')

    def test_send_mail_without_vars(self):
        self.message.send()
        self.assert_sailthru_called()
        data = self.get_api_call_data()
        self.assertEqual(data['template'], 'Email Template')
        self.assertEqual(data['email'], 'foo@bar.com')
        self.assertEqual(data['vars'], {})

    def test_send_email_with_vars(self):
        params = {'foo': 'bar'}
        message = SailthruEmailMessage('foo@bar.com', 'Email Template', vars=params)
        message.send()
        self.assert_sailthru_called()
        data = self.get_api_call_data()
        self.assertEqual(data['template'], 'Email Template')
        self.assertEqual(data['email'], 'foo@bar.com')
        self.assertEqual(data['vars'], {'vars': params})

    def test_send_email_bad_email(self):
        message = SailthruEmailMessage('foo@barcom', 'Email Template')
        self.mock_api_post.return_value = self.MockResponse(
            is_ok=False,
            json=json.dumps({'errormsg': 'Invalid email: foo@barcom', 'error': 11}))
        with self.assertRaises(SailthruClientError):
            message.send()



__all__ = (
    'override_settings',
)

try:
    from django.test.utils import override_settings

except ImportError:
    # Back-port override_settings from Django 1.4
    # https://github.com/django/django/blob/stable/1.4.x/django/test/utils.py
    from django.conf import settings, UserSettingsHolder
    from django.utils.functional import wraps

    class override_settings(object):
        """
        Acts as either a decorator, or a context manager. If it's a decorator it
        takes a function and returns a wrapped function. If it's a contextmanager
        it's used with the ``with`` statement. In either event entering/exiting
        are called before and after, respectively, the function/block is executed.
        """
        def __init__(self, **kwargs):
            self.options = kwargs
            self.wrapped = settings._wrapped

        def __enter__(self):
            self.enable()

        def __exit__(self, exc_type, exc_value, traceback):
            self.disable()

        def __call__(self, test_func):
            from django.test import TransactionTestCase
            if isinstance(test_func, type) and issubclass(test_func, TransactionTestCase):
                original_pre_setup = test_func._pre_setup
                original_post_teardown = test_func._post_teardown

                def _pre_setup(innerself):
                    self.enable()
                    original_pre_setup(innerself)

                def _post_teardown(innerself):
                    original_post_teardown(innerself)
                    self.disable()
                test_func._pre_setup = _pre_setup
                test_func._post_teardown = _post_teardown
                return test_func
            else:
                @wraps(test_func)
                def inner(*args, **kwargs):
                    with self:
                        return test_func(*args, **kwargs)
            return inner

        def enable(self):
            override = UserSettingsHolder(settings._wrapped)
            for key, new_value in self.options.items():
                setattr(override, key, new_value)
            settings._wrapped = override
            # No setting_changed signal in Django 1.3
            # for key, new_value in self.options.items():
            #     setting_changed.send(sender=settings._wrapped.__class__,
            #                          setting=key, value=new_value)

        def disable(self):
            settings._wrapped = self.wrapped
            # No setting_changed signal in Django 1.3
            # for key in self.options:
            #     new_value = getattr(settings, key, None)
            #     setting_changed.send(sender=settings._wrapped.__class__,
            #                          setting=key, value=new_value)



from django.core.mail import EmailMessage


class SailthruEmailMessage(EmailMessage):
    def __init__(self, email_address, template_name, connection=None, **kwargs):
        self.data = {
            "template": template_name,
            "email": email_address,
            "vars": kwargs
        }
        self.connection = connection

    def send(self, fail_silently=False):
        """Sends the email message."""
        return self.get_connection(fail_silently).send_messages([self])



from mock import patch

from django.test import TestCase
from .utils import override_settings


@override_settings(SAILTHRU_API_KEY="FAKE_API_KEY_FOR_TESTING",
                   SAILTHRU_API_SECRET="FAKE_API_SECRET_FOR_TESTING",
                   EMAIL_BACKEND="djsailthru.mail.backends.base.SailthruEmailBackend")
class SailthruBackendMockTestCase(TestCase):

    class MockResponse:
        """requests.post return value mock sufficient for SailthruEmailBackend"""
        def __init__(self, is_ok=True, get_status_code=200, json=None):
            self._status_code = get_status_code
            self._json = json if json is not None else ['']
            self._is_ok = is_ok

        def json(self):
            return self._json

        def is_ok(self):
            return self._is_ok

        def get_body(self):
            return self._json

        def get_status_code(self):
            return self._get_status_code

    def setUp(self):
        self.patch = patch('sailthru.sailthru_client.SailthruClient.api_post', autospec=True)
        self.mock_api_post = self.patch.start()
        self.mock_api_post.return_value = self.MockResponse()

    def assert_sailthru_called(self):
        if self.mock_api_post.call_args is None:
            raise AssertionError("Sailthru API was not called")

        (args, kwargs) = self.mock_api_post.call_args
        if args[1] != 'send':
            raise AssertionError('sailthru api was not called with `send` method')
        if not isinstance(args[2], dict):
            raise AssertionError('sailthru was called with out vars')

    def get_api_call_data(self):
        """Returns the data posted to the Sailthru API.

        Fails test if API wasn't called.
        """
        if self.mock_api_post.call_args is None:
            raise AssertionError("Sailthru API was not called")
        (args, kwargs) = self.mock_api_post.call_args
        try:
            post_data = args[2]
        except IndexError:
            raise AssertionError('sailthru was called with out vars')
        return post_data



from django.core.mail.backends.base import BaseEmailBackend
from django.conf import settings

from sailthru import sailthru_client as sc
from sailthru.sailthru_error import SailthruClientError

from djsailthru.mail.message import SailthruEmailMessage


class SailthruEmailBackend(BaseEmailBackend):
    def __init__(self, *args, **kwargs):
        super(SailthruEmailBackend, self).__init__(*args, **kwargs)

        self.sailthru_client = sc.SailthruClient(
            settings.SAILTHRU_API_KEY,
            settings.SAILTHRU_API_SECRET,
        )

    def send_messages(self, email_messages):
        if not email_messages:
            return

        num_sent = 0
        for message in email_messages:
            sent = self._send(message)
            if sent:
                num_sent += 1

    def _send(self, email_message):
        if not isinstance(email_message, SailthruEmailMessage):
            raise TypeError('`email_message is not of type `SailthruEmailMessage')

        response = self.sailthru_client.api_post("send", email_message.data)
        email_message.sailthru_response = None
        if response.is_ok():
            email_message.sailthru_response = response.get_body()
            return True
        else:
            if not self.fail_silently:
                raise SailthruClientError(response.get_body())
            else:
                return False



#!/usr/bin/env python
from setuptools import setup, find_packages
import os

__doc__ = """
App for using Sailthru as a django email backend.
"""


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='djsailthru',
    version="0.0.2",
    description=__doc__,
    long_description=read('README.rst'),
    url="https://github.com/fxdgear/djsailthru",
    author="Nick Lang",
    author_email='nick@nicklang.com',
    packages=[package for package in find_packages() if package.startswith('djsailthru')],
    install_requires=[
        'Django>=1.4',
        'sailthru-client>=2.1.3'
    ],
    zip_safe=False,
    include_package_data=True,
)
